{% extends 'dj_accounts/auth_base.html' %}
{% load i18n static %}

{% block page_title %}
{% translate "Password Reset" %}
{% endblock %}

{% block content %}
<div class="card">
  <div class="card-body login-card-body">
    <p class="login-box-msg">
      {% translate 'We’ve emailed you instructions for setting your password, if an account exists with the email you
      entered. You should receive them shortly.' %}
    </p>
    <p class="login-box-msg">
      {% translate 'If you don’t receive an email, please make sure you’ve entered the address you registered with, and
      check your spam folder.' %}
    </p>
  </div>
</div>
{% endblock %}
