{% load i18n %}
<div class="icheck-primary">
  <input id="email-remember" name="remember_me" type="checkbox">
  <label for="email-remember">
    {% translate 'I agree to the ' %}<a href="#">{% translate 'trems' %}</a>.
  </label>
</div>