{% extends 'dj_accounts/auth_base.html' %}
{% load i18n static %}

{% block page_title %}
{% translate "Password Reset" %}
{% endblock %}

{% block content %}
<div class="card">
  <div class="card-body login-card-body">
    <p class="login-box-msg">
      {% translate "Your password has been set. You may go ahead and log in now." %}
    </p>
    <a class="btn btn-link btn-block" href="{{ login_url }}">{% translate "Log in" %}</a>
  </div>
</div>
{% endblock %}

