<div class="mb-3">
  <label class="sr-only" for="{{ form.last_name.id_for_label }}">{{ form.last_name.label }}</label>
  <div class="input-group">
    <input class="form-control{% if form.last_name.errors %} is-invalid{% endif %}"
           id="{{ form.last_name.id_for_label }}"
           name="{{ form.last_name.html_name }}"
           placeholder="{{ form.last_name.label }}"
           type="text"
           value="{{ form.last_name.value|default_if_none:'' }}">
    <div class="input-group-append">
      <span class="input-group-text">
              <span class="fas fa-user"></span>
      </span>
    </div>
  </div>
  {% if form.last_name.errors %}
  <div class="invalid-feedback" id="{{ form.last_name.id_for_label }}-feedback">
    {{ form.last_name.errors.as_ul }}
  </div>
  {% endif %}
</div>